@@ -18,19 +18,26 @@ THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABI
 CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS
 IN THE SOFTWARE.
 """
-from __future__ import unicode_literals
 import wx
 import wx.lib.mixins.listctrl as listmix
 import functools
-from ... import util
+from .. import util
 from collections import OrderedDict
+import locale
+from ..settings import Settings
 
 MINIMUM_COL_SIZE = 100
 COLUMN_SAMPLE_SIZE = 100
 USE_SAMPLE_SIZE = True
 
 
-class DummyLock(object):
+def cmp(a, b):
+    """Compare."""
+
+    return (a > b) - (a < b)
+
+
+class DummyLock:
     """A dummy lock that does nothing."""
 
     def __enter__(self):
@@ -45,15 +52,18 @@ class DummyLock(object):
 class DynamicList(wx.ListCtrl, listmix.ColumnSorterMixin):
     """Dynamic list."""
 
-    def __init__(self, parent, columns, single_sel=True):
+    def __init__(self, parent, columns, single_sel=True, virtual_list=None):
         """Initialize the base class `DynamicList` object."""
 
+        if virtual_list is None:
+            virtual_list = []
+
         flags = wx.LC_REPORT | wx.LC_VIRTUAL
 
         if single_sel:
             flags |= wx.LC_SINGLE_SEL
 
-        super(DynamicList, self).__init__(
+        super().__init__(
             parent, wx.ID_ANY, wx.DefaultPosition, wx.DefaultSize,
             style=flags
         )
@@ -61,7 +71,7 @@ class DynamicList(wx.ListCtrl, listmix.ColumnSorterMixin):
             # Select all
             self.set_keybindings(
                 [
-                    (wx.ACCEL_CMD if util.platform() == "osx" else wx.ACCEL_CTRL, ord('A'), self.select_all)
+                    (wx.ACCEL_CMD if util.platform() == "macos" else wx.ACCEL_CTRL, ord('A'), self.select_all)
                 ]
             )
 
@@ -72,12 +82,15 @@ class DynamicList(wx.ListCtrl, listmix.ColumnSorterMixin):
         self.resize_complete = False
         self.wait = DummyLock()
         self.column_count = len(columns)
+        self.col2virt = {x: x for x in range(self.column_count)}
+        self.virt2col = {v: k for k, v in self.col2virt.items()}
+        self.setup_virtual(virtual_list)
         self.headers = columns
         self.itemDataMap = OrderedDict()
         self.first_resize = True
         self.size_sample = COLUMN_SAMPLE_SIZE
         self.widest_cell = [MINIMUM_COL_SIZE] * self.column_count
-        self.EnableAlternateRowColours()
+        self.EnableAlternateRowColours(enable=Settings.get_alt_list_color())
         self.dc = wx.ClientDC(self)
         self.dc.SetFont(self.GetFont())
         self.last_idx_sized = -1
@@ -85,12 +98,72 @@ class DynamicList(wx.ListCtrl, listmix.ColumnSorterMixin):
         self.setup_columns()
         self.itemIndexMap = []
 
+    def setup_virtual(self, virtual_list):
+        """Setup virtual list."""
+
+        virtual_list = [x for x in virtual_list if (0 <= x < self.column_count)]
+        if len(virtual_list) != len(set(virtual_list)):
+            new_list = []
+            for x in virtual_list:
+                if x not in new_list:
+                    new_list.append(x)
+            virtual_list = new_list
+        if len(virtual_list) > self.column_count:
+            virtual_list = [x for x in virtual_list if x < self.column_count]
+        if len(virtual_list) < self.column_count - 1:
+            for x in range(self.column_count):
+                if x not in virtual_list:
+                    virtual_list.append(x)
+
+        for real, virt in enumerate(virtual_list, 0):
+            self.col2virt[real] = virt
+        self.virt2col = {v: k for k, v in self.col2virt.items()}
+        self.virtual_list = virtual_list
+        return virtual_list
+
+    def update_virtual(self, virtual_list):
+        """Update virtual."""
+
+        self.complete = False
+        column, order = self.GetSortState()
+        if column != -1:
+            column = self.get_real_col(column)
+
+        old_widest = {}
+        for virt, size in enumerate(self.widest_cell):
+            old_widest[self.get_real_col(virt)] = size
+
+        virtual_list = self.setup_virtual(virtual_list)
+        for virt in range(self.column_count):
+            self.widest_cell[virt] = old_widest[self.get_real_col(virt)]
+
+        self.DeleteAllColumns()
+        self.setup_columns()
+        self.SetColumnCount(self.column_count)
+        if column != -1:
+            self.SortListItems(self.get_virt_col(column), order)
+        self.Refresh()
+        self.virtual_list = virtual_list
+        self.complete = True
+        return virtual_list
+
+    def get_virt_col(self, index):
+        """Get virtual column from real column."""
+
+        return self.virt2col.get(index, index)
+
+    def get_real_col(self, index):
+        """Get real column from virtual column."""
+
+        return self.col2virt.get(index, index)
+
     def set_hidden_columns(self, hidden):
         """Set hidden columns."""
 
         self.hidden_columns = set(hidden)
         for i in range(0, self.column_count):
-            if i in self.hidden_columns:
+            real = self.get_real_col(i)
+            if real in self.hidden_columns:
                 self.SetColumnWidth(i, 0)
             elif self.GetColumnWidth(i) == 0:
                 self.SetColumnWidth(i, MINIMUM_COL_SIZE)
@@ -161,7 +234,6 @@ class DynamicList(wx.ListCtrl, listmix.ColumnSorterMixin):
             last_width = w
             if w > 0:
                 last_column = i
-        total_width -= last_width
 
         if total_width < (self.GetSize()[0] - 20) and last_column > -1:
             self.SetColumnWidth(last_column, last_width + self.GetSize()[0] - total_width)
@@ -170,9 +242,15 @@ class DynamicList(wx.ListCtrl, listmix.ColumnSorterMixin):
         """Setup columns."""
 
         for x in range(0, self.column_count):
-            self.InsertColumn(x, self.headers[x])
+            real = self.get_real_col(x)
+            lw = self.dc.GetFullTextExtent(self.headers[real])[0]
+            width = lw + 30
+            if width > self.widest_cell[x]:
+                self.widest_cell[x] = width
+            self.InsertColumn(x, self.headers[real])
         for i in range(0, self.column_count):
-            if i not in self.hidden_columns:
+            real = self.get_real_col(i)
+            if real not in self.hidden_columns:
                 self.SetColumnWidth(i, self.widest_cell[i])
             else:
                 self.SetColumnWidth(i, 0)
@@ -202,7 +280,7 @@ class DynamicList(wx.ListCtrl, listmix.ColumnSorterMixin):
     def get_map_item(self, idx, col=0, absolute=False):
         """Get attribute in in item map entry and the given index."""
 
-        return self.itemDataMap[self.itemIndexMap[idx] if not absolute else idx][col]
+        return self.itemDataMap[self.itemIndexMap[idx] if not absolute else idx][self.get_real_col(col)]
 
     def reset_list(self):
         """Reset the list."""
@@ -215,6 +293,7 @@ class DynamicList(wx.ListCtrl, listmix.ColumnSorterMixin):
         self.SetItemCount(0)
         self.size_sample = COLUMN_SAMPLE_SIZE
         self.widest_cell = [MINIMUM_COL_SIZE] * self.column_count
+        self.SetColumnCount(self.column_count)
         self.setup_columns()
         self.Refresh()
 
@@ -224,7 +303,8 @@ class DynamicList(wx.ListCtrl, listmix.ColumnSorterMixin):
         self.SetItemCount(len(self.itemDataMap))
         if not self.resize_complete:
             for i in range(0, self.column_count):
-                if i not in self.hidden_columns:
+                real = self.get_real_col(i)
+                if real not in self.hidden_columns:
                     self.SetColumnWidth(i, self.widest_cell[i])
                 else:
                     self.SetColumnWidth(i, 0)
@@ -241,14 +321,49 @@ class DynamicList(wx.ListCtrl, listmix.ColumnSorterMixin):
     def get_item_text(self, idx, col, absolute=False):
         """Return the text for the given item and col."""
 
-        return util.to_ustr(self.itemDataMap[self.itemIndexMap[idx] if not absolute else idx][col])
+        return util.to_ustr(self.itemDataMap[self.itemIndexMap[idx] if not absolute else idx][self.get_real_col(col)])
 
     def GetSecondarySortValues(self, col, key1, key2):
-        """Get secondary sort values."""
+        """
+        Get secondary sort values.
+
+        Virtual columns are handled in `__ColumnSorter`.
+        """
 
         sscol = 1 if col == 0 else 0
         return (self.itemDataMap[key1][sscol], self.itemDataMap[key2][sscol])
 
+    def GetColumnSorter(self):
+        """Returns a callable object to be used for comparing column values when sorting."""
+
+        return self._custom_sorter
+
+    def _custom_sorter(self, key1, key2):
+        """Custom virtual columns sorter."""
+
+        col = self._col
+        ascending = self._colSortFlag[col]
+        real = self.get_real_col(col)
+        item1 = self.itemDataMap[key1][real]
+        item2 = self.itemDataMap[key2][real]
+
+        # Internationalization of string sorting with locale module
+        if isinstance(item1, str) and isinstance(item2, str):
+            cmpVal = locale.strcoll(item1, item2)
+        elif isinstance(item1, bytes) or isinstance(item2, bytes):
+            cmpVal = locale.strcoll(str(item1), str(item2))
+        else:
+            cmpVal = cmp(item1, item2)
+
+        # If the items are equal, then pick something else to make the sort value unique
+        if cmpVal == 0:
+            cmpVal = cmp(*self.GetSecondarySortValues(col, key1, key2))
+
+        if ascending:
+            return cmpVal
+        else:
+            return -cmpVal
+
     def SortItems(self, sorter=None):
         """Sort items."""
 
