@@ -18,7 +18,6 @@ THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABI
 CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS
 IN THE SOFTWARE.
 """
-from __future__ import unicode_literals
 import time
 import wx
 import decimal
@@ -28,9 +27,10 @@ from .dynamic_lists import DynamicList, USE_SAMPLE_SIZE
 from ..actions import fileops
 from ..localization import _
 from .. import data
-from ... import util
+from .. import util
 from ..settings import Settings
 from ..actions import checksum
+from ..dialogs.generic_dialogs import yesno
 
 CONTENT_PATH = 0
 CONTENT_LINE = 1
@@ -99,7 +99,131 @@ class ContextMenu(wx.Menu):
                         parent.Bind(wx.EVT_MENU, i[1], item)
 
 
-class ResultFileList(DynamicList):
+class CommonOperationsMixin:
+    """Handle common operations."""
+
+    def on_enter_window(self, event):
+        """Reset last moused over item tracker on mouse entering the window."""
+
+        self.last_moused = (-1, "")
+        event.Skip()
+
+    def on_leave_window(self, event):
+        """Clear timed status when cursor leaves window."""
+
+        self.main_window.m_statusbar.cancel_timed_status()
+
+    def create_image_list(self):
+        """Create the image list."""
+
+        self.images = wx.ImageList(16, 16)
+        self.tint = data.RGBA(self.GetForegroundColour().Get())
+        self.doc = self.images.Add(data.get_bitmap('doc.png'))
+        self.bin = self.images.Add(data.get_bitmap('binary.png'))
+        self.sort_up = self.images.Add(data.get_bitmap('arrow_up.png', tint=self.tint, alpha=0.3))
+        self.sort_down = self.images.Add(data.get_bitmap('arrow_down.png', tint=self.tint, alpha=0.3))
+        self.AssignImageList(self.images, wx.IMAGE_LIST_SMALL)
+
+    def on_col_hide_click(self, event, col=-1):
+        """Handle column hide click."""
+
+        hidden = list(self.hidden_columns)
+
+        if not event.IsChecked():
+            if col not in hidden:
+                hidden.append(col)
+        elif col in hidden:
+            del hidden[hidden.index(col)]
+        self.set_hidden_columns(hidden)
+        self.save_hidden_columns(self.hidden_columns)
+
+    def on_arrange_click(self, event):
+        """Handle arranging columns."""
+
+        from ..dialogs.column_dialog import ColumnDialog
+
+        dlg = ColumnDialog(self.main_window, self.virtual_list, self.headers)
+        dlg.ShowModal()
+        if dlg.changed:
+            virtual_list = self.update_virtual(dlg.virtual_columns)
+            self.save_pos_columns(virtual_list)
+        dlg.Destroy()
+
+    def on_checksum(self, event, h, target):
+        """Handle checksum event."""
+
+        from ..dialogs.checksum_dialog import ChecksumDialog
+
+        dlg = ChecksumDialog(self.main_window, h, target)
+        dlg.ShowModal()
+        dlg.Destroy()
+
+    def on_delete_files(self, event, recycle):
+        """Delete files in the list control."""
+
+        from ..dialogs.delete_dialog import DeleteDialog
+
+        if not yesno(self.RECYCLE if recycle else self.DELETE):
+            return
+
+        remove = None
+        files = self.get_selected_files()
+        dlg = DeleteDialog(self.main_window, files, recycle)
+        dlg.ShowModal()
+        dlg.Destroy()
+
+        self.main_window.m_result_list.deselect_all(None)
+        self.main_window.m_result_file_list.deselect_all(None)
+
+        # Remove entries from table that no longer exist while rebuilding table index
+        if self.main_window.m_result_list.GetItemCount():
+            remove = set()
+            new_index = []
+            for index in self.main_window.m_result_list.itemIndexMap:
+                data = self.main_window.m_result_list.itemDataMap[index]
+                if not os.path.exists(os.path.join(data[CONTENT_PATH][1], data[CONTENT_PATH][0])):
+                    del self.main_window.m_result_list.itemDataMap[index]
+                    remove.add(data[CONTENT_KEY])
+                else:
+                    new_index.append(index)
+            self.main_window.m_result_list.itemIndexMap = new_index
+            self.main_window.m_result_list.SetItemCount(len(self.main_window.m_result_list.itemDataMap))
+
+        new_index = []
+        for index in self.main_window.m_result_file_list.itemIndexMap:
+            if (remove is None and not os.path.exists(index)) or (remove and index in remove):
+                del self.main_window.m_result_file_list.itemDataMap[index]
+            else:
+                new_index.append(index)
+        self.main_window.m_result_file_list.itemIndexMap = new_index
+        self.main_window.m_result_file_list.SetItemCount(len(self.main_window.m_result_file_list.itemDataMap))
+
+    def on_col_rclick(self, event):
+        """Handle col right click."""
+
+        menu_entries = [None] * len(self.headers)
+        for index, header in enumerate(self.headers):
+            menu_entries[self.get_virt_col(index)] = (
+                header,
+                functools.partial(self.on_col_hide_click, col=index),
+                True,
+                index not in self.hidden_columns
+            )
+
+        menu_entries.append(
+            (
+                self.ARRANGE_COLUMNS,
+                functools.partial(self.on_arrange_click),
+                self.complete
+            )
+        )
+
+        menu = ContextMenu(menu_entries)
+        self.PopupMenu(menu)
+        menu.Destroy()
+
+
+class ResultFileList(CommonOperationsMixin, DynamicList):
     """Result file list."""
 
     def __init__(self, parent):
@@ -108,7 +232,7 @@ class ResultFileList(DynamicList):
         self.localize()
         self.international = False
 
-        super(ResultFileList, self).__init__(
+        super().__init__(
             parent,
             [
                 self.FILE,
@@ -120,14 +244,17 @@ class ResultFileList(DynamicList):
                 self.MODIFIED,
                 self.CREATED
             ],
-            False
+            False,
+            self.get_pos_columns()
         )
         self.last_moused = (-1, "")
         self.Bind(wx.EVT_LEFT_DCLICK, self.on_dclick)
         self.Bind(wx.EVT_MOTION, self.on_motion)
+        self.Bind(wx.EVT_LEAVE_WINDOW, self.on_leave_window)
         self.Bind(wx.EVT_ENTER_WINDOW, self.on_enter_window)
         self.Bind(wx.EVT_RIGHT_DOWN, self.on_rclick)
         self.Bind(wx.EVT_LIST_COL_RIGHT_CLICK, self.on_col_rclick)
+        self.set_hidden_columns(self.get_hidden_columns())
 
     def localize(self):
         """Translate strings."""
@@ -143,7 +270,7 @@ class ResultFileList(DynamicList):
         self.EDITOR_LABEL = _("Open in Editor")
         self.REVEAL_LABEL = {
             "windows": _("Reveal in Explorer"),
-            "osx": _("Reveal in Finder"),
+            "macos": _("Reveal in Finder"),
             "linux": _("Reveal in File Manager")
         }
         self.COPY_NAME = _("Copy File Names")
@@ -151,16 +278,29 @@ class ResultFileList(DynamicList):
         self.CHECKSUM_LABEL = _("Checksum")
         self.DELETE_LABEL = _("Delete")
         self.RECYCLE_LABEL = _("Send to Trash")
+        self.ARRANGE_COLUMNS = _("Reorder columns")
+        self.DELETE = _("Are you sure you want to delete the files?")
+        self.RECYCLE = _("Are you sure you want to recycle the files?")
 
-    def create_image_list(self):
-        """Create the image list."""
+    def get_hidden_columns(self):
+        """Get hidden columns."""
 
-        self.images = wx.ImageList(16, 16)
-        self.doc = self.images.Add(data.get_bitmap('doc.png'))
-        self.bin = self.images.Add(data.get_bitmap('binary.png'))
-        self.sort_up = self.images.Add(data.get_bitmap('su.png'))
-        self.sort_down = self.images.Add(data.get_bitmap('sd.png'))
-        self.AssignImageList(self.images, wx.IMAGE_LIST_SMALL)
+        return Settings.get_hide_cols_file()
+
+    def save_hidden_columns(self, hidden):
+        """Save hidden columns."""
+
+        Settings.set_hide_cols_file(hidden)
+
+    def get_pos_columns(self):
+        """Get position of columns."""
+
+        return Settings.get_pos_cols_file()
+
+    def save_pos_columns(self, pos):
+        """Save position of columns."""
+
+        Settings.set_pos_cols_file(pos)
 
     def set_international_time(self, enable):
         """Enable or disable international time output."""
@@ -203,12 +343,6 @@ class ResultFileList(DynamicList):
                     obj.match.colno
                 )
 
-    def on_enter_window(self, event):
-        """Reset last moused over item tracker on mouse entering the window."""
-
-        self.last_moused = (-1, "")
-        event.Skip()
-
     def on_motion(self, event):
         """Display full file path in status bar on item mouseover."""
 
@@ -218,25 +352,28 @@ class ResultFileList(DynamicList):
             if item != -1:
                 actual_item = self.itemIndexMap[item]
                 if actual_item != self.last_moused[0]:
-                    d = self.itemDataMap[actual_item]
-                    self.last_moused = (actual_item, os.path.join(d[FILE_PATH], d[FILE_NAME]))
+                    data = self.itemDataMap[actual_item]
+                    self.last_moused = (actual_item, os.path.join(data[FILE_PATH], data[FILE_NAME]))
                 self.main_window.m_statusbar.set_timed_status(self.last_moused[1])
+            else:
+                self.main_window.m_statusbar.cancel_timed_status()
         event.Skip()
 
     def get_item_text(self, item, col, absolute=False):
         """Return the text for the given item and col."""
 
+        real = self.get_real_col(col)
         if not absolute:
             item = self.itemIndexMap[item]
-        if col == FILE_SIZE:
+        if real == FILE_SIZE:
             return '%.2fKB' % round(self.itemDataMap[item][FILE_SIZE], 2)
-        elif col in (FILE_MOD, FILE_CRE):
+        elif real in (FILE_MOD, FILE_CRE):
             if self.international:
-                return time.strftime("%Y-%m-%d %H:%M:%S", time.localtime(self.itemDataMap[item][col]))
+                return time.strftime("%Y-%m-%d %H:%M:%S", time.localtime(self.itemDataMap[item][real]))
             else:
-                return time.strftime("%b %d, %Y, %I:%M:%S %p", time.localtime(self.itemDataMap[item][col]))
+                return time.strftime("%b %d, %Y, %I:%M:%S %p", time.localtime(self.itemDataMap[item][real]))
         else:
-            return util.to_ustr(self.itemDataMap[item][col])
+            return util.to_ustr(self.itemDataMap[item][real])
 
     def OnGetItemImage(self, item):
         """Override method to get the image for the given item."""
@@ -253,11 +390,12 @@ class ResultFileList(DynamicList):
         # Sample the first "size_sample" to determine
         # column width for when table first loads
         if idx <= self.last_idx_sized or not USE_SAMPLE_SIZE:
-            text = self.get_item_text(idx, FILE_MATCH, True)
+            virt = self.get_virt_col(FILE_MATCH)
+            text = self.get_item_text(idx, virt, True)
             lw = self.dc.GetFullTextExtent(text)[0]
             width = lw + 30
-            if width > self.widest_cell[FILE_MATCH]:
-                self.widest_cell[FILE_MATCH] = width
+            if width > self.widest_cell[virt]:
+                self.widest_cell[virt] = width
 
     def on_dclick(self, event):
         """Open file at in editor with optional line and column argument."""
@@ -274,11 +412,11 @@ class ResultFileList(DynamicList):
         if item != -1:
             target = None
             with self.wait:
-                filename = self.GetItem(item, col=FILE_NAME).GetText()
-                path = self.GetItem(item, col=FILE_PATH).GetText()
+                filename = self.GetItem(item, col=self.get_virt_col(FILE_NAME)).GetText()
+                path = self.GetItem(item, col=self.get_virt_col(FILE_PATH)).GetText()
                 target = os.path.join(path, filename)
-                line = str(self.get_map_item(item, col=FILE_LINE))
-                col = str(self.get_map_item(item, col=FILE_COL))
+                line = self.get_map_item(item, col=self.get_virt_col(FILE_LINE))
+                col = self.get_map_item(item, col=self.get_virt_col(FILE_COL))
             if target:
                 fileops.open_editor(target, line, col)
         else:
@@ -286,11 +424,11 @@ class ResultFileList(DynamicList):
             while item != -1:
                 target = None
                 with self.wait:
-                    filename = self.GetItem(item, col=FILE_NAME).GetText()
-                    path = self.GetItem(item, col=FILE_PATH).GetText()
+                    filename = self.GetItem(item, col=self.get_virt_col(FILE_NAME)).GetText()
+                    path = self.GetItem(item, col=self.get_virt_col(FILE_PATH)).GetText()
                     target = os.path.join(path, filename)
-                    line = str(self.get_map_item(item, col=FILE_LINE))
-                    col = str(self.get_map_item(item, col=FILE_COL))
+                    line = self.get_map_item(item, col=self.get_virt_col(FILE_LINE))
+                    col = self.get_map_item(item, col=self.get_virt_col(FILE_COL))
                 if target:
                     fileops.open_editor(target, line, col)
                 item = self.GetNextSelected(item)
@@ -304,9 +442,9 @@ class ResultFileList(DynamicList):
         while item != -1:
             with self.wait:
                 if col == FILE_NAME:
-                    copy_bfr.append(self.GetItem(item, col=FILE_NAME).GetText())
+                    copy_bfr.append(self.GetItem(item, col=self.get_virt_col(FILE_NAME)).GetText())
                 elif col == FILE_PATH:
-                    copy_bfr.append(self.GetItem(item, col=FILE_PATH).GetText())
+                    copy_bfr.append(self.GetItem(item, col=self.get_virt_col(FILE_PATH)).GetText())
             item = self.GetNextSelected(item)
 
         if copy_bfr:
@@ -325,8 +463,8 @@ class ResultFileList(DynamicList):
             item = self.GetNextItem(-1)
             while item != -1:
                 if self.IsSelected(item):
-                    filename = self.GetItem(item, col=FILE_NAME).GetText()
-                    path = self.GetItem(item, col=FILE_PATH).GetText()
+                    filename = self.GetItem(item, col=self.get_virt_col(FILE_NAME)).GetText()
+                    path = self.GetItem(item, col=self.get_virt_col(FILE_PATH)).GetText()
                     files.add(os.path.join(path, filename))
                 item = self.GetNextItem(item)
         return sorted(list(files))
@@ -341,8 +479,8 @@ class ResultFileList(DynamicList):
             pos = event.GetPosition()
             item = self.HitTestSubItem(pos)[0]
             if item != -1:
-                filename = self.GetItem(item, col=FILE_NAME).GetText()
-                path = self.GetItem(item, col=FILE_PATH).GetText()
+                filename = self.GetItem(item, col=self.get_virt_col(FILE_NAME)).GetText()
+                path = self.GetItem(item, col=self.get_virt_col(FILE_PATH)).GetText()
                 target = os.path.join(path, filename)
                 selected = self.IsSelected(item)
                 select_count = self.GetSelectedItemCount()
@@ -361,7 +499,7 @@ class ResultFileList(DynamicList):
 
             hash_entries = []
             for h in checksum.VALID_HASH:
-                hash_entries.append((h, functools.partial(self.main_window.on_checksum, h=h, target=target)))
+                hash_entries.append((h, functools.partial(self.on_checksum, h=h, target=target)))
 
             # Open menu
             menu = ContextMenu(
@@ -375,12 +513,12 @@ class ResultFileList(DynamicList):
                     None,
                     (
                         self.DELETE_LABEL,
-                        functools.partial(self.main_window.on_delete_files, recycle=False, listctrl=self),
+                        functools.partial(self.on_delete_files, recycle=False),
                         self.complete
                     ),
                     (
                         self.RECYCLE_LABEL,
-                        functools.partial(self.main_window.on_delete_files, recycle=True, listctrl=self),
+                        functools.partial(self.on_delete_files, recycle=True),
                         self.complete
                     )
                 ]
@@ -389,39 +527,8 @@ class ResultFileList(DynamicList):
             menu.Destroy()
         event.Skip()
 
-    def on_col_hide_click(self, event, col=-1):
-        """Handle column hide click."""
 
-        hidden = list(self.hidden_columns)
-
-        if not event.IsChecked():
-            if col not in hidden:
-                hidden.append(col)
-        elif col in hidden:
-            del hidden[hidden.index(col)]
-        self.set_hidden_columns(hidden)
-        Settings.set_hide_cols_file(list(self.hidden_columns))
-
-    def on_col_rclick(self, event):
-        """Handle column right click."""
-
-        menu = ContextMenu(
-            [
-                (self.FILE, functools.partial(self.on_col_hide_click, col=0), True, 0 not in self.hidden_columns),
-                (self.MATCHES, functools.partial(self.on_col_hide_click, col=1), True, 1 not in self.hidden_columns),
-                (self.EXTENSION, functools.partial(self.on_col_hide_click, col=2), True, 2 not in self.hidden_columns),
-                (self.SIZE, functools.partial(self.on_col_hide_click, col=3), True, 3 not in self.hidden_columns),
-                (self.PATH, functools.partial(self.on_col_hide_click, col=4), True, 4 not in self.hidden_columns),
-                (self.ENCODING, functools.partial(self.on_col_hide_click, col=5), True, 5 not in self.hidden_columns),
-                (self.MODIFIED, functools.partial(self.on_col_hide_click, col=6), True, 6 not in self.hidden_columns),
-                (self.CREATED, functools.partial(self.on_col_hide_click, col=7), True, 7 not in self.hidden_columns),
-            ]
-        )
-        self.PopupMenu(menu)
-        menu.Destroy()
-
-
-class ResultContentList(DynamicList):
+class ResultContentList(CommonOperationsMixin, DynamicList):
     """Result content list."""
 
     def __init__(self, parent):
@@ -429,7 +536,7 @@ class ResultContentList(DynamicList):
 
         self.localize()
 
-        super(ResultContentList, self).__init__(
+        super().__init__(
             parent,
             [
                 self.FILE,
@@ -438,14 +545,17 @@ class ResultContentList(DynamicList):
                 self.EXTENSION,
                 self.CONTEXT
             ],
-            False
+            False,
+            self.get_pos_columns()
         )
         self.last_moused = (-1, "")
         self.Bind(wx.EVT_LEFT_DCLICK, self.on_dclick)
         self.Bind(wx.EVT_MOTION, self.on_motion)
+        self.Bind(wx.EVT_LEAVE_WINDOW, self.on_leave_window)
         self.Bind(wx.EVT_ENTER_WINDOW, self.on_enter_window)
         self.Bind(wx.EVT_RIGHT_DOWN, self.on_rclick)
         self.Bind(wx.EVT_LIST_COL_RIGHT_CLICK, self.on_col_rclick)
+        self.set_hidden_columns(self.get_hidden_columns())
 
     def localize(self):
         """Translate strings."""
@@ -458,7 +568,7 @@ class ResultContentList(DynamicList):
         self.EDITOR_LABEL = _("Open in Editor")
         self.REVEAL_LABEL = {
             "windows": _("Reveal in Explorer"),
-            "osx": _("Reveal in Finder"),
+            "macos": _("Reveal in Finder"),
             "linux": _("Reveal in File Manager")
         }
         self.COPY_NAME = _("Copy File Names")
@@ -467,9 +577,36 @@ class ResultContentList(DynamicList):
         self.CHECKSUM_LABEL = _("Checksum")
         self.DELETE_LABEL = _("Delete")
         self.RECYCLE_LABEL = _("Send to Trash")
+        self.ARRANGE_COLUMNS = _("Reorder Columns")
+        self.DELETE = _("Are you sure you want to delete the files?")
+        self.RECYCLE = _("Are you sure you want to recycle the files?")
+
+    def get_hidden_columns(self):
+        """Get hidden columns."""
+
+        return Settings.get_hide_cols_content()
+
+    def save_hidden_columns(self, hidden):
+        """Save hidden columns."""
+
+        Settings.set_hide_cols_content(hidden)
+
+    def get_pos_columns(self):
+        """Get position of columns."""
+
+        return Settings.get_pos_cols_content()
+
+    def save_pos_columns(self, pos):
+        """Save position of columns."""
+
+        Settings.set_pos_cols_content(pos)
 
     def GetSecondarySortValues(self, col, key1, key2):
-        """Get secondary sort values."""
+        """
+        Get secondary sort values.
+
+        Virtual columns are handled in `__ColumnSorter`.
+        """
 
         if col == CONTENT_LINE:
             return (self.itemDataMap[key1][CONTENT_PATH], self.itemDataMap[key2][CONTENT_PATH])
@@ -481,22 +618,6 @@ class ResultContentList(DynamicList):
                 (self.itemDataMap[key2][CONTENT_PATH], self.itemDataMap[key2][CONTENT_LINE])
             )
 
-    def create_image_list(self):
-        """Create the image list."""
-
-        self.images = wx.ImageList(16, 16)
-        self.doc = self.images.Add(data.get_bitmap('doc.png'))
-        self.bin = self.images.Add(data.get_bitmap('binary.png'))
-        self.sort_up = self.images.Add(data.get_bitmap('su.png'))
-        self.sort_down = self.images.Add(data.get_bitmap('sd.png'))
-        self.AssignImageList(self.images, wx.IMAGE_LIST_SMALL)
-
-    def on_enter_window(self, event):
-        """Reset last moused over item tracker on mouse entering the window."""
-
-        self.last_moused = (-1, "")
-        event.Skip()
-
     def on_motion(self, event):
         """Display full file path in status bar on item mouseover."""
 
@@ -509,17 +630,20 @@ class ResultContentList(DynamicList):
                     pth = self.itemDataMap[actual_item][CONTENT_PATH]
                     self.last_moused = (actual_item, os.path.join(pth[1], pth[0]))
                 self.main_window.m_statusbar.set_timed_status(self.last_moused[1])
+            else:
+                self.main_window.m_statusbar.cancel_timed_status()
         event.Skip()
 
     def get_item_text(self, item, col, absolute=False):
         """Return the text for the given item and col."""
 
+        real = self.get_real_col(col)
         if not absolute:
             item = self.itemIndexMap[item]
-        if col == CONTENT_PATH:
+        if real == CONTENT_PATH:
             return util.to_ustr(self.itemDataMap[item][CONTENT_PATH][0])
         else:
-            return util.to_ustr(self.itemDataMap[item][col])
+            return util.to_ustr(self.itemDataMap[item][real])
 
     def increment_match_count(self, idx):
         """Increment the match count of the given item."""
@@ -530,11 +654,12 @@ class ResultContentList(DynamicList):
         # Sample the first "size_sample" to determine
         # column width for when table first loads
         if idx <= self.last_idx_sized or not USE_SAMPLE_SIZE:
-            text = self.get_item_text(idx, CONTENT_MATCH, True)
+            virt = self.get_virt_col(CONTENT_MATCH)
+            text = self.get_item_text(idx, virt, True)
             lw = self.dc.GetFullTextExtent(text)[0]
             width = lw + 30
-            if width > self.widest_cell[CONTENT_MATCH]:
-                self.widest_cell[CONTENT_MATCH] = width
+            if width > self.widest_cell[virt]:
+                self.widest_cell[virt] = width
 
     def set_match(self, obj):
         """Set the match."""
@@ -568,12 +693,12 @@ class ResultContentList(DynamicList):
             pos = event.GetPosition()
             item = self.HitTestSubItem(pos)[0]
             if item != -1:
-                filename = self.GetItem(item, col=CONTENT_PATH).GetText()
-                line = self.GetItem(item, col=CONTENT_LINE).GetText()
-                file_row = self.get_map_item(item, col=CONTENT_KEY)
-                col = str(self.get_map_item(item, col=CONTENT_COL))
+                filename = self.GetItem(item, col=self.get_virt_col(CONTENT_PATH)).GetText()
+                line = self.get_map_item(item, col=self.get_virt_col(CONTENT_LINE))
+                file_row = self.get_map_item(item, col=self.get_virt_col(CONTENT_KEY))
+                col = self.get_map_item(item, col=self.get_virt_col(CONTENT_COL))
                 path = self.main_window.m_result_file_list.get_map_item(
-                    file_row, col=FILE_PATH, absolute=True
+                    file_row, col=self.main_window.m_result_file_list.get_virt_col(FILE_PATH), absolute=True
                 )
                 fileops.open_editor(os.path.join(os.path.normpath(path), filename), line, col)
         event.Skip()
@@ -584,14 +709,14 @@ class ResultContentList(DynamicList):
         if item != -1:
             target = None
             with self.wait:
-                file_row = self.get_map_item(item, col=CONTENT_KEY)
-                filename = self.GetItem(item, col=CONTENT_PATH).GetText()
+                file_row = self.get_map_item(item, col=self.get_virt_col(CONTENT_KEY))
+                filename = self.GetItem(item, col=self.get_virt_col(CONTENT_PATH)).GetText()
                 path = self.main_window.m_result_file_list.get_map_item(
-                    file_row, col=FILE_PATH, absolute=True
+                    file_row, col=self.main_window.m_result_file_list.get_virt_col(FILE_PATH), absolute=True
                 )
                 target = os.path.join(path, filename)
-                line = self.GetItem(item, col=CONTENT_LINE).GetText()
-                col = str(self.get_map_item(item, col=CONTENT_COL))
+                line = self.get_map_item(item, col=self.get_virt_col(CONTENT_LINE))
+                col = self.get_map_item(item, col=self.get_virt_col(CONTENT_COL))
             if target:
                 fileops.open_editor(target, line, col)
         else:
@@ -600,14 +725,14 @@ class ResultContentList(DynamicList):
             while item != -1:
                 target = None
                 with self.wait:
-                    file_row = self.get_map_item(item, col=CONTENT_KEY)
-                    filename = self.GetItem(item, col=CONTENT_PATH).GetText()
+                    file_row = self.get_map_item(item, col=self.get_virt_col(CONTENT_KEY))
+                    filename = self.GetItem(item, col=self.get_virt_col(CONTENT_PATH)).GetText()
                     path = self.main_window.m_result_file_list.get_map_item(
-                        file_row, col=FILE_PATH, absolute=True
+                        file_row, col=self.main_window.m_result_file_list.get_virt_col(FILE_PATH), absolute=True
                     )
                     target = os.path.join(path, filename)
-                    line = self.GetItem(item, col=CONTENT_LINE).GetText()
-                    col = str(self.get_map_item(item, col=CONTENT_COL))
+                    line = self.get_map_item(item, col=self.get_virt_col(CONTENT_LINE))
+                    col = self.get_map_item(item, col=self.get_virt_col(CONTENT_COL))
                 if target and target not in found:
                     found.add(target)
                     fileops.open_editor(target, line, col)
@@ -623,16 +748,16 @@ class ResultContentList(DynamicList):
             with self.wait:
                 if col == CONTENT_PATH:
                     if from_file_tab:
-                        file_row = self.get_map_item(item, col=CONTENT_KEY)
+                        file_row = self.get_map_item(item, col=self.get_virt_col(CONTENT_KEY))
                         path = self.main_window.m_result_file_list.get_map_item(
-                            file_row, col=FILE_PATH, absolute=True
+                            file_row, col=self.main_window.m_result_file_list.get_virt_col(FILE_PATH), absolute=True
                         )
-                        filename = self.GetItem(item, col=CONTENT_PATH).GetText()
+                        filename = self.GetItem(item, col=self.get_virt_col(CONTENT_PATH)).GetText()
                         copy_bfr.append(os.path.join(path, filename))
                     else:
-                        copy_bfr.append(self.GetItem(item, col=CONTENT_PATH).GetText())
+                        copy_bfr.append(self.GetItem(item, col=self.get_virt_col(CONTENT_PATH)).GetText())
                 else:
-                    copy_bfr.append(self.GetItem(item, col=CONTENT_TEXT).GetText())
+                    copy_bfr.append(self.GetItem(item, col=self.get_virt_col(CONTENT_TEXT)).GetText())
             item = self.GetNextSelected(item)
 
         if copy_bfr:
@@ -651,11 +776,11 @@ class ResultContentList(DynamicList):
             item = self.GetNextItem(-1)
             while item != -1:
                 if self.IsSelected(item):
-                    file_row = self.get_map_item(item, col=CONTENT_KEY)
+                    file_row = self.get_map_item(item, col=self.get_virt_col(CONTENT_KEY))
                     path = self.main_window.m_result_file_list.get_map_item(
-                        file_row, col=FILE_PATH, absolute=True
+                        file_row, col=self.main_window.m_result_file_list.get_virt_col(FILE_PATH), absolute=True
                     )
-                    filename = self.GetItem(item, col=CONTENT_PATH).GetText()
+                    filename = self.GetItem(item, col=self.get_virt_col(CONTENT_PATH)).GetText()
                     files.add(os.path.join(path, filename))
                 item = self.GetNextItem(item)
         return sorted(list(files))
@@ -670,10 +795,10 @@ class ResultContentList(DynamicList):
             pos = event.GetPosition()
             item = self.HitTestSubItem(pos)[0]
             if item != -1:
-                file_row = self.get_map_item(item, col=CONTENT_KEY)
-                filename = self.GetItem(item, col=CONTENT_PATH).GetText()
+                file_row = self.get_map_item(item, col=self.get_virt_col(CONTENT_KEY))
+                filename = self.GetItem(item, col=self.get_virt_col(CONTENT_PATH)).GetText()
                 path = self.main_window.m_result_file_list.get_map_item(
-                    file_row, col=FILE_PATH, absolute=True
+                    file_row, col=self.main_window.m_result_file_list.get_virt_col(FILE_PATH), absolute=True
                 )
                 target = os.path.join(path, filename)
                 selected = self.IsSelected(item)
@@ -693,7 +818,7 @@ class ResultContentList(DynamicList):
 
             hash_entries = []
             for h in checksum.VALID_HASH:
-                hash_entries.append((h, functools.partial(self.main_window.on_checksum, h=h, target=target)))
+                hash_entries.append((h, functools.partial(self.on_checksum, h=h, target=target)))
 
             # Open menu
             menu = ContextMenu(
@@ -708,12 +833,12 @@ class ResultContentList(DynamicList):
                     None,
                     (
                         self.DELETE_LABEL,
-                        functools.partial(self.main_window.on_delete_files, recycle=False, listctrl=self),
+                        functools.partial(self.on_delete_files, recycle=False),
                         self.complete
                     ),
                     (
                         self.RECYCLE_LABEL,
-                        functools.partial(self.main_window.on_delete_files, recycle=True, listctrl=self),
+                        functools.partial(self.on_delete_files, recycle=True),
                         self.complete
                     )
                 ]
@@ -721,31 +846,3 @@ class ResultContentList(DynamicList):
             self.PopupMenu(menu, pos)
             menu.Destroy()
         event.Skip()
-
-    def on_col_hide_click(self, event, col=-1):
-        """Handle column hide click."""
-
-        hidden = list(self.hidden_columns)
-
-        if not event.IsChecked():
-            if col not in hidden:
-                hidden.append(col)
-        elif col in hidden:
-            del hidden[hidden.index(col)]
-        self.set_hidden_columns(hidden)
-        Settings.set_hide_cols_content(list(self.hidden_columns))
-
-    def on_col_rclick(self, event):
-        """Handle col right click."""
-
-        menu = ContextMenu(
-            [
-                (self.FILE, functools.partial(self.on_col_hide_click, col=0), True, 0 not in self.hidden_columns),
-                (self.LINE, functools.partial(self.on_col_hide_click, col=1), True, 1 not in self.hidden_columns),
-                (self.MATCHES, functools.partial(self.on_col_hide_click, col=2), True, 2 not in self.hidden_columns),
-                (self.EXTENSION, functools.partial(self.on_col_hide_click, col=3), True, 3 not in self.hidden_columns),
-                (self.CONTEXT, functools.partial(self.on_col_hide_click, col=4), True, 4 not in self.hidden_columns)
-            ]
-        )
-        self.PopupMenu(menu)
-        menu.Destroy()
