@@ -32,7 +32,7 @@ class ErrorList(DynamicList):
 
         self.localize()
 
-        super(ErrorList, self).__init__(
+        super().__init__(
             parent,
             [
                 self.ERROR,
@@ -52,20 +52,22 @@ class ErrorList(DynamicList):
         """Create image list."""
 
         self.images = wx.ImageList(16, 16)
+        self.tint = data.RGBA(self.GetForegroundColour().Get())
         self.error_symbol = self.images.Add(data.get_bitmap('error.png'))
-        self.sort_up = self.images.Add(data.get_bitmap('su.png'))
-        self.sort_down = self.images.Add(data.get_bitmap('sd.png'))
+        self.sort_up = self.images.Add(data.get_bitmap('arrow_up.png', tint=self.tint, alpha=0.3))
+        self.sort_down = self.images.Add(data.get_bitmap('arrow_down.png', tint=self.tint, alpha=0.3))
         self.AssignImageList(self.images, wx.IMAGE_LIST_SMALL)
 
     def get_item_text(self, item, col, absolute=False):
         """Return the text for the given item and col."""
 
+        real = self.get_real_col(col)
         if not absolute:
             item = self.itemIndexMap[item]
-        if col == 0:
-            return self.itemDataMap[item][col][0]
+        if real == 0:
+            return self.itemDataMap[item][real][0]
         else:
-            return self.itemDataMap[item][col]
+            return self.itemDataMap[item][real]
 
     def on_dclick(self, event):
         """Open file at in editor with optional line and column argument."""
@@ -73,7 +75,7 @@ class ErrorList(DynamicList):
         pos = event.GetPosition()
         item = self.HitTestSubItem(pos)[0]
         if item != -1:
-            file_name = self.get_map_item(item, col=1)
-            full_error = ''.join(reversed(self.get_map_item(item, col=0)))
+            file_name = self.get_map_item(item, col=self.get_virt_col(1))
+            full_error = ''.join(reversed(self.get_map_item(item, col=self.get_virt_col(0))))
             self.GetParent().GetParent().show_error("%s\n%s" % (file_name, full_error))
         event.Skip()
