@@ -1,10 +1,11 @@
 """Data resource library."""
-from __future__ import unicode_literals
 import os
 import codecs
 import base64
 from wx.lib.embeddedimage import PyEmbeddedImage
-from ... import util
+from .. import util
+from ..util.rgba import RGBA  # noqa: F401
+from ..util import images
 
 RESOURCE_PATH = os.path.abspath(os.path.dirname(__file__))
 
@@ -27,20 +28,25 @@ def get_file(file_name, raw=False):
     return text
 
 
-def get_image(file_name, b64=False):
+def get_image(file_name, b64=False, tint=None, alpha=None):
     """Get the image as a `PyEmbeddedImage`."""
     icon = b''
     resource = os.path.join(RESOURCE_PATH, file_name)
     if os.path.exists(resource):
         try:
             with open(resource, "rb") as f:
-                icon = base64.b64encode(f.read())
+                if tint:
+                    icon = base64.b64encode(images.tint(f.read(), tint, alpha))
+                elif alpha is not None:
+                    icon = base64.b64encode(images.transparency(f.read(), alpha))
+                else:
+                    icon = base64.b64encode(f.read())
         except Exception:
             pass
     return PyEmbeddedImage(icon) if not b64 else util.to_ustr(icon)
 
 
-def get_bitmap(file_name):
+def get_bitmap(file_name, tint=None, alpha=None):
     """
     Get bitmap.
 
@@ -55,8 +61,8 @@ def get_bitmap(file_name):
     and load a separate 2X size for macOS retina.  But this is an okay work around for now.
     """
 
-    image = get_image(file_name).GetImage()
-    if util.platform() == "osx":
+    image = get_image(file_name, tint=tint, alpha=alpha).GetImage()
+    if util.platform() == "macos":
         bm = image.ConvertToBitmap()
         bm.SetSize(
             (
