@@ -18,7 +18,6 @@ THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABI
 CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS
 IN THE SOFTWARE.
 """
-from __future__ import unicode_literals
 import codecs
 import json
 import os
@@ -26,14 +25,15 @@ import copy
 from filelock import FileLock
 from ..app import custom_app
 from ..app.custom_app import debug, debug_struct, error
-from ..generic_dialogs import errormsg
+from ..dialogs.generic_dialogs import errormsg
 from .. import localization
 from ..localization import _
 from .. import data
 from .. import notify
+from ..notify.util import which
 from ... import rumcore
 from ...rumcore import text_decode
-from ... import util
+from .. import util
 
 DEV_MODE = False
 SETTINGS_FILE = "rummage_dev.settings" if DEV_MODE else "rummage.settings"
@@ -41,22 +41,36 @@ CACHE_FILE = "rummage_dev.cache" if DEV_MODE else "rummage.cache"
 LOG_FILE = "rummage.log"
 FIFO = "rummage.fifo"
 
-SETTINGS_FMT = '2.3.1'
+SETTINGS_FMT = '2.6.0'
 CACHE_FMT = '2.0.0'
 
 NOTIFY_STYLES = {
-    "osx": ["default", "growl"],
+    "macos": ["default", "growl"],
     "windows": ["default", "growl"],
     "linux": ["default", "growl"]
 }
 
+NOTIFY_PLAYERS = {
+    "macos": ["afplay"],
+    "windows": ["windows"],
+    "linux": ["paplay", "aplay", "play"]
+}
+
+NOTIFY_EXT = {
+    "afplay": ['.wav', '.mp3', '.aiff'],
+    "windows": ['.wav'],
+    "paplay": ['.wav', '.mp3', '.ogg'],
+    "aplay": ['.wav', '.mp3'],
+    "play": ['.wav', '.mp3'],
+}
+
 BACKUP_FILE = 0
 BACKUP_FOLDER = 1
 
 DEFAULT_SETTINGS = {
     "__format__": SETTINGS_FMT,
-    "current_version": (0, 0, 0, 'final', 0, 0),
     "alert_enabled": True,
+    "alt_list_color": True,
     "backup_ext": "rum-bak",
     "backup_folder": ".rum-bak",
     "backup_type": BACKUP_FILE,
@@ -64,7 +78,9 @@ DEFAULT_SETTINGS = {
     "chains": {},
     "check_prerelease": False,
     "check_updates": False,
+    "current_version": (0, 0, 0, 'final', 0, 0),
     "editor": "",
+    "encoding_options": copy.deepcopy(text_decode.DEFAULT_ENCODING_OPTIONS),
     "extmatch": False,
     "file_case_sensitive": False,
     "full_exclude_path": False,
@@ -72,21 +88,25 @@ DEFAULT_SETTINGS = {
     "globstar": False,
     "hide_cols_content": [],
     "hide_cols_file": [],
-    "hide_limit": False,
     "international_time": False,
     "locale": "en_US",
+    "matchbase": False,
     "notify_enabled": True,
     "notify_method": "default",
+    "notify_player": NOTIFY_PLAYERS[util.platform()][0],
+    "notify_sound": "",
+    "pattern_limit": 1000,
+    "pos_cols_content": [],
+    "pos_cols_file": [],
     "regex_mode": rumcore.RE_MODE,
     "regex_version": 0,
     "saved_searches": {},
     "single_instance": False,
-    "term_notifier": "",
-    "encoding_options": copy.deepcopy(text_decode.DEFAULT_ENCODING_OPTIONS)
+    "term_notifier": ""
 }
 
 
-class Settings(object):
+class Settings:
     """Handle settings."""
 
     filename = None
@@ -122,6 +142,12 @@ class Settings(object):
         debug_struct(cls.cache)
         cls.init_notify(True)
 
+    @classmethod
+    def get_available_players(cls):
+        """Get the available players."""
+
+        return NOTIFY_PLAYERS[util.platform()][:]
+
     @classmethod
     def get_settings(cls):
         """Get the entire settings object."""
@@ -197,6 +223,27 @@ class Settings(object):
             value = text_decode.CHARDET_DEFAULT
         return value
 
+    @classmethod
+    def set_alt_list_color(cls, value):
+        """Set alternate list row color."""
+
+        cls.reload_settings()
+        cls._set_alt_list_color(value)
+        cls.save_settings()
+
+    @classmethod
+    def _set_alt_list_color(cls, value):
+        """Set alternate list row color."""
+
+        cls.settings["alt_list_color"] = value
+
+    @classmethod
+    def get_alt_list_color(cls):
+        """Get preference of alternative list row color."""
+
+        cls.reload_settings()
+        return cls.settings['alt_list_color']
+
     @classmethod
     def _set_encoding_ext(cls, values):
         """Set encoding options."""
@@ -285,27 +332,6 @@ class Settings(object):
         cls.reload_settings()
         return cls.settings.get('regex_version', 0)
 
-    @classmethod
-    def get_hide_limit(cls):
-        """Get hide limit setting."""
-
-        cls.reload_settings()
-        return cls.settings.get("hide_limit", False)
-
-    @classmethod
-    def set_hide_limit(cls, hide):
-        """Set hide limit setting."""
-
-        cls.reload_settings()
-        cls._set_hide_limit(hide)
-        cls.save_settings()
-
-    @classmethod
-    def _set_hide_limit(cls, hide):
-        """Set hide limit setting."""
-
-        cls.settings["hide_limit"] = hide
-
     @classmethod
     def get_hide_cols_file(cls):
         """Get hide cols file."""
@@ -348,6 +374,48 @@ class Settings(object):
 
         cls.settings["hide_cols_content"] = hide
 
+    @classmethod
+    def get_pos_cols_file(cls):
+        """Get position cols file."""
+
+        cls.reload_settings()
+        return cls.settings.get("pos_cols_file", [])
+
+    @classmethod
+    def set_pos_cols_file(cls, pos):
+        """Set position cols file."""
+
+        cls.reload_settings()
+        cls._set_pos_cols_file(pos)
+        cls.save_settings()
+
+    @classmethod
+    def _set_pos_cols_file(cls, pos):
+        """Set position cols file."""
+
+        cls.settings["pos_cols_file"] = pos
+
+    @classmethod
+    def get_pos_cols_content(cls):
+        """Get position cols content."""
+
+        cls.reload_settings()
+        return cls.settings.get("pos_cols_content", [])
+
+    @classmethod
+    def set_pos_cols_content(cls, pos):
+        """Set position cols content."""
+
+        cls.reload_settings()
+        cls._set_pos_cols_content(pos)
+        cls.save_settings()
+
+    @classmethod
+    def _set_pos_cols_content(cls, pos):
+        """Set position cols content."""
+
+        cls.settings["pos_cols_content"] = pos
+
     @classmethod
     def get_international_time(cls):
         """Get international time setting."""
@@ -462,7 +530,7 @@ class Settings(object):
             cls.log = os.path.join(folder, LOG_FILE)
             cls.fifo = os.path.join(folder, '\\\\.\\pipe\\rummage')
             cls.config_folder = folder
-        elif platform == "osx":
+        elif platform == "macos":
             old_folder = os.path.expanduser("~/Library/Application Support/Rummage")
             folder = os.path.expanduser("~/.Rummage")
             if os.path.exists(old_folder) and not os.path.exists(folder):
@@ -565,9 +633,6 @@ class Settings(object):
             backup_type = cls.settings.get('backup_type')
             cls.settings["backup_type"] = 0 if backup_type is None or backup_type is False else 1
 
-            # Update format
-            cls.settings["__format__"] = SETTINGS_FMT
-
         if settings_format < (2, 2, 0):
             if 'editor' in cls.settings and isinstance(cls.settings['editor'], (list, tuple)):
                 cls.settings['editor'] = ""
@@ -583,6 +648,10 @@ class Settings(object):
                         updated = True
                         cls.settings[k][k1] = copy.copy(v1)
 
+        if settings_format < tuple([int(x) for x in SETTINGS_FMT.split('.')]):
+            updated = True
+            cls.settings["__format__"] = SETTINGS_FMT
+
         if updated:
             cls.save_settings()
 
@@ -665,7 +734,7 @@ class Settings(object):
                 cls.cache = cache
 
     @classmethod
-    def get_editor(cls, filename="{$file}", line="{$line}", col="{$col}"):
+    def get_editor(cls, filename=None, line=None, col=None):
         """Get editor command and replace file, line, and col symbols."""
 
         cls.reload_settings()
@@ -675,6 +744,9 @@ class Settings(object):
         if isinstance(editor, dict):
             editor = editor.get(util.platform(), "")
 
+        if filename is None or line is None or col is None:
+            return editor
+
         if isinstance(editor, (list, tuple)):
             # Handle old list format
             return [
@@ -688,8 +760,15 @@ class Settings(object):
             ]
         else:
             # New string format
-            filename = filename.replace('"', '\\"')
-            return editor.replace("{$file}", filename).replace("{$line}", str(line)).replace("{$col}", str(col))
+            return editor.replace(
+                "{$file}", filename.replace('"', '\\"')
+            ).replace(
+                "{$line}", str(line)
+            ).replace(
+                "{$col}", str(col)
+            ).replace(
+                "{$col0}", str(col - 1)
+            )
 
     @classmethod
     def set_editor(cls, editor):
@@ -900,7 +979,7 @@ class Settings(object):
 
         if util.platform() == "windows":
             img = icon
-        elif util.platform() == "osx":
+        elif util.platform() == "macos":
             img = icns
         else:
             img = png
@@ -908,7 +987,10 @@ class Settings(object):
         notify.setup_notifications(
             "Rummage",
             img,
-            (notifier, None)
+            term_notify=notifier,
+            sender=None,
+            sound=cls.get_notify_sound(),
+            sound_player=cls.get_notify_player()
         )
         notify.setup_growl_notifications("Rummage", growl_png)
         notify.enable_growl(cls.get_notify_method() == "growl" and notify.has_growl())
@@ -990,6 +1072,88 @@ class Settings(object):
 
         cls.settings['term_notifier'] = value
 
+    @classmethod
+    def get_notify_sound(cls):
+        """Get notifier sound."""
+
+        cls.reload_settings()
+        sound = cls.settings.get('notify_sound', '')
+        player = cls._get_notify_player()
+        if sound is None or not os.path.exists(sound) or not os.path.isfile(sound):
+            sound = ''
+        if os.path.splitext(sound)[1].lower() not in NOTIFY_EXT[player]:
+            sound = ''
+        return sound
+
+    @classmethod
+    def set_notify_sound(cls, value):
+        """Set notifier sound."""
+
+        cls.reload_settings()
+        cls._set_notify_sound(value)
+        cls.save_settings()
+        cls.init_notify()
+
+    @classmethod
+    def _set_notify_sound(cls, sound):
+        """Set notifier sound."""
+
+        player = cls._get_notify_player()
+        if sound is None or not os.path.exists(sound) or not os.path.isfile(sound):
+            sound = ''
+        if os.path.splitext(sound)[1].lower() not in NOTIFY_EXT[player]:
+            sound = ''
+        cls.settings['notify_sound'] = sound
+
+    @classmethod
+    def get_notify_player(cls):
+        """Get notifier player."""
+
+        cls.reload_settings()
+        player = cls._get_notify_player()
+        return player
+
+    @classmethod
+    def validate_player(cls, player):
+        """Validate player."""
+
+        if player is None or player not in NOTIFY_PLAYERS[util.platform()]:
+            player = NOTIFY_PLAYERS[util.platform()][0]
+        if util.platform() == "linux" and not which(player):
+            player = None
+            for p in NOTIFY_PLAYERS['linux']:
+                if which(p):
+                    player = p
+                    break
+        if player is None:
+            # None exist, so just go with the first
+            player = NOTIFY_PLAYERS[util.platform()][0]
+        return player
+
+    @classmethod
+    def _get_notify_player(cls):
+        """Get notifier player."""
+
+        player = cls.settings.get('notify_player')
+        player = cls.validate_player(player)
+        return player
+
+    @classmethod
+    def set_notify_player(cls, player):
+        """Set notifier player."""
+
+        cls.reload_settings()
+        cls._set_notify_player(player)
+        cls.save_settings()
+        cls.init_notify()
+
+    @classmethod
+    def _set_notify_player(cls, player):
+        """Set notifier player."""
+
+        player = cls.validate_player(player)
+        cls.settings['notify_player'] = player
+
     @classmethod
     def get_search_setting(cls, key, default):
         """Get search history setting from cache."""
@@ -998,11 +1162,41 @@ class Settings(object):
         return cls.cache.get(key, default)
 
     @classmethod
-    def add_search_settings(cls, history, toggles, strings):
-        """Add search settings to cache (more like history...but whatever)."""
+    def set_toggles(cls, toggles, save=True):
+        """Set toggles."""
 
-        cls.reload_settings()
-        debug(history)
+        if save:
+            cls.reload_settings()
+        for t in toggles:
+            key = t[0]
+            value = t[1]
+            if value is None:
+                continue
+            cls.cache[key] = value
+        if save:
+            cls.save_cache()
+
+    @classmethod
+    def set_strings(cls, strings, save=True):
+        """Set strings."""
+
+        if save:
+            cls.reload_settings()
+        for s in strings:
+            key = s[0]
+            value = s[1]
+            if value is None:
+                continue
+            cls.cache[key] = value
+        if save:
+            cls.save_cache()
+
+    @classmethod
+    def set_history(cls, history, save=True):
+        """Set history."""
+
+        if save:
+            cls.reload_settings()
         for i in history:
             key = i[0]
             value = i[1]
@@ -1017,19 +1211,18 @@ class Settings(object):
             if len(values) > (20 if value != "" else 21):
                 del values[-1]
             cls.cache[key] = values
-        for t in toggles:
-            key = t[0]
-            value = t[1]
-            if value is None:
-                continue
-            cls.cache[key] = value
-        for s in strings:
-            key = s[0]
-            value = s[1]
-            if value is None:
-                continue
-            cls.cache[key] = value
+        if save:
+            cls.save_cache()
 
+    @classmethod
+    def add_search_settings(cls, history, toggles, strings):
+        """Add search settings to cache (more like history...but whatever)."""
+
+        cls.reload_settings()
+        debug(history)
+        cls.set_history(history, False)
+        cls.set_toggles(toggles, False)
+        cls.set_strings(strings, False)
         cls.save_cache()
 
     @classmethod
@@ -1240,6 +1433,31 @@ class Settings(object):
         cls.reload_settings()
         return cls.settings.get('brace_expansion', False)
 
+    @classmethod
+    def get_pattern_limit(cls):
+        """Get pattern_limit."""
+
+        cls.reload_settings()
+        value = cls.settings.get('pattern_limit', DEFAULT_SETTINGS['pattern_limit'])
+        if not isinstance(value, int):
+            value = DEFAULT_SETTINGS['pattern_limit']
+        return value
+
+    @classmethod
+    def set_pattern_limit(cls, value):
+        """Set pattern_limit."""
+
+        cls.reload_settings()
+        cls._set_pattern_limit(value)
+        cls.save_settings()
+
+    @classmethod
+    def _set_pattern_limit(cls, value):
+        """Set pattern_limit."""
+
+        if isinstance(value, int):
+            cls.settings['pattern_limit'] = value
+
     @classmethod
     def _set_file_case_sensitive(cls, value):
         """Set file_case_sensitive."""
@@ -1324,6 +1542,27 @@ class Settings(object):
         cls.reload_settings()
         return cls.settings.get('globstar', False)
 
+    @classmethod
+    def _set_matchbase(cls, value):
+        """Set `matchbase`."""
+
+        cls.settings['matchbase'] = value
+
+    @classmethod
+    def set_matchbase(cls, value):
+        """Set `matchbase`."""
+
+        cls.reload_settings()
+        cls._set_matchbase(value)
+        cls.save_settings()
+
+    @classmethod
+    def get_matchbase(cls):
+        """Get `matchbase`."""
+
+        cls.reload_settings()
+        return cls.settings.get('matchbase', False)
+
     @classmethod
     def import_settings(cls, obj):
         """Import settings."""
@@ -1362,6 +1601,10 @@ class Settings(object):
             cls._set_full_file_path(obj['full_file_path'])
         if 'globstar' in obj:
             cls._set_globstar(obj['globstar'])
+        if 'matchbase' in obj:
+            cls._set_matchbase(obj['matchbase'])
+        if 'pattern_limit' in obj:
+            cls._set_pattern_limit(obj['pattern_limit'])
 
         # Notifications
         update_notify = False
@@ -1375,6 +1618,9 @@ class Settings(object):
         if 'term_notifier' in obj:
             cls._set_term_notifier(obj['term_notifier'])
             update_notify = True
+        if 'notify_sound' in obj:
+            cls._set_notify_sound(obj['notify_sound'])
+            update_notify = True
 
         # Editor
         if 'editor' in obj:
@@ -1388,16 +1634,22 @@ class Settings(object):
         if 'locale' in obj:
             cls._set_language(obj['locale'])
 
-        # Hide limit panel
-        if 'hide_limit' in obj:
-            cls._set_hide_limit(obj['hide_limit'])
-
         # Hide columns
         if 'hide_cols_file' in obj:
             cls._set_hide_cols_file(obj['hide_cols_file'])
         if 'hide_cols_content' in obj:
             cls._set_hide_cols_content(obj['hide_cols_content'])
 
+        # Alternate row colors for lists
+        if 'alt_list_color' in obj:
+            cls._set_alt_list_color(obj['alt_list_color'])
+
+        # Position columns
+        if 'pos_cols_file' in obj:
+            cls._set_pos_cols_file(obj['pos_cols_file'])
+        if 'pos_cols_content' in obj:
+            cls._set_pos_cols_content(obj['pos_cols_content'])
+
         # International time
         if 'international_time' in obj:
             cls._set_international_time(obj['international_time'])
